program main
    use mpi

    integer :: ierror, rank, nprocs, num_args, tslength, nts, rrank
    integer, parameter :: n = 100
    integer, dimension(n) :: m, rm
    integer, dimension(MPI_STATUS_SIZE) :: mpi_status
    character(len=12), dimension(:), allocatable :: args
    character(len=12) :: infile

    num_args = command_argument_count()
    allocate(args(num_args))

    if (num_args < 1)  then
        write(*,*) 'Usage: mpirun -np <nprocs> ./main <arg1>...'
        stop
    end if

    call get_command_argument(1, infile)

    open(unit=10, file=infile, status='old', action='read', iostat=ierror)
    if (ierror /= 0) then
        write(*,*) 'Error opening file ', infile
        stop
    end if

    read(10, *) tslength, nts

    ! Initialize MPI environment
    call MPI_Init(ierror)
    
    ! Get process rank
    call MPI_Comm_rank(MPI_COMM_WORLD, rank, ierror)
    
    ! Get total number of processes
    call MPI_Comm_size(MPI_COMM_WORLD, nprocs, ierror)

    do i = 1, nts
        m = [ (j, j = rank, rank + n) ]

        call my_sleep(tslength)

        call MPI_Send(m,  n, MPI_INTEGER, mod(rank + i, nprocs), 0, MPI_COMM_WORLD, ierror)

        rrank = mod(rank - i, nprocs)
        call MPI_Recv(rm, n, MPI_INTEGER, rrank, 0, MPI_COMM_WORLD, mpi_status, ierror)

        call MPI_Barrier(MPI_COMM_WORLD, ierror)

        if (rank == 0) then
            write(*, *) 'Timestep:', i
        end if
    end do
        
    ! Finalize MPI environment
    call MPI_Finalize(ierror)
end program main

subroutine my_sleep(seconds)
    integer, intent(in) :: seconds

    call sleep(seconds)
end subroutine my_sleep